package chaincodetest

import (
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"github.com/hyperledger/fabric/protos/peer"
	"github.com/stretchr/testify/mock"
	"github.com/hyperledger/fabric/protos/ledger/queryresult"
)

// ChaincodeStubInterfaceAdapterMock chaincode adapter mock interface
type StubInterfaceAdapterMock struct {
	mock.Mock
}

// GetState
func (mock *StubInterfaceAdapterMock) GetState(key string) ([]byte, error) {
	args := mock.Called(key)
	if args.Get(0) == nil && args.Get(1) == nil {
		return nil, nil
	}
	return args.Get(0).([]byte), args.Error(1)
}

// PutState
func (mock *StubInterfaceAdapterMock) PutState(key string, value []byte) error {
	args := mock.Called(key, value)
	return args.Error(0)
}

// GetQueryResult
func (mock *StubInterfaceAdapterMock) GetQueryResult(query string) (shim.StateQueryIteratorInterface, error) {
	args := mock.Called(query)
	return args.Get(0).(shim.StateQueryIteratorInterface), args.Error(1)
}

func (stub *StubInterfaceAdapterMock) InvokeChaincode(chaincodeName string, args [][]byte, channel string) peer.Response {
	return stub.Called(chaincodeName, args, channel).Get(0).(peer.Response)
}

func (stub *StubInterfaceAdapterMock) GetTxID() string {
	return "test_tx_id"
}

// stateQueryIteratorInterfaceMock query result iterator
type stateQueryIteratorInterfaceMock struct {
	mock.Mock
}

func (query *stateQueryIteratorInterfaceMock) Close() error {
	return nil
}

func (query *stateQueryIteratorInterfaceMock) Next() (*queryresult.KV, error) {
	return query.Called().Get(0).(*queryresult.KV), query.Called().Error(1)
}

func (query *stateQueryIteratorInterfaceMock) HasNext() bool {
	return query.Called().Get(0).(bool)
}
