package function

import (
	"fmt"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/tools"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/actions"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/model"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"errors"
)

type UserPreferenceLoader_Type func(externalAccountId string, adapter chaincode.StubInterfaceAdapter) (*model.UserPreference, error)

func UserPreferenceLoader_Function(externalAccountId string, adapter chaincode.StubInterfaceAdapter) (*model.UserPreference, error) {
	logMsg := fmt.Sprintf("userPreferenceLoaderFunction (externalAccountId: %s)", externalAccountId)
	logger.Error(logMsg)

	resp := tools.SignedInvokeChaincode(adapter, invoke.GetIntAccountByExtAccountChannel, actions.Actions().UserPreferenceCC().GetIntAccountByExtAccount, requests.Id{Id: externalAccountId})

	if resp.Status == 404 {
		logger.Debug("UserPreference is not found")
		return nil, nil
	}

	// We got error status
	if resp.Status != int32(shim.OK) {
		logger.Errorf("Failed to invoke getIntAccountByExtAccount from userPreferenceCC, with following error message %s", resp.Message)
		return nil, errors.New(resp.Message)
	}
	// No error, account verified
	var userPreference model.UserPreference
	err := json.Unmarshal(resp.Payload, &userPreference)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to deserialize user UserPreference, due to %s", err)
		logger.Error(errMsg)
		return nil, err
	}
	return &userPreference, nil
}

